@@ -4,7 +4,7 @@
 # @param address_pack packaged addresses object
 # Vintage must be defined if return = 'geographies'
 batch_census <- function(unique_addresses,
-     return_type = 'locations', timeout = 20, full_results = FALSE, custom_query = list(), api_url = NULL,
+     api_options = list(), timeout = 20, full_results = FALSE, custom_query = list(), api_url = NULL,
      lat = 'lat', long = 'long', verbose = FALSE, ...) {
   
   if (!'street' %in% names(unique_addresses) & (!'address' %in% names(unique_addresses))) {
@@ -13,12 +13,12 @@ batch_census <- function(unique_addresses,
   
   location_cols <- c('id', 'input_address', 'match_indicator', 'match_type','matched_address', 
           'coords', 'tiger_line_id', 'tiger_side')
-  return_cols <- switch(return_type,
+  return_cols <- switch(api_options[["census_return_type"]],
           'locations' = location_cols,
           'geographies' = c(location_cols, c('state_fips', 'county_fips', 'census_tract', 'census_block'))
   )
   
-  if (is.null(api_url)) api_url <- get_census_url(return_type, 'addressbatch')
+  if (is.null(api_url)) api_url <- get_census_url(api_options[["census_return_type"]], 'addressbatch')
   
   num_addresses <- nrow(unique_addresses)
 
@@ -36,7 +36,7 @@ batch_census <- function(unique_addresses,
   utils::write.table(input_df, tmp, row.names = FALSE, col.names = FALSE, sep = ',', na = '')
   
   # Construct query
-  # NOTE - request will fail if vintage and benchmark are invalid for return_type = 'geographies'
+  # NOTE - request will fail if vintage and benchmark are invalid for census_return_type = 'geographies'
   query_parameters <- get_api_query('census', custom_parameters = custom_query)
   if (verbose == TRUE) display_query(api_url, query_parameters)
   
@@ -46,7 +46,7 @@ batch_census <- function(unique_addresses,
   
   # force certain geographies columns to be read in as character instead of numeric
   # to preserve leading zeros (for FIPS codes)
-  column_classes <- ifelse(return_type == 'geographies',
+  column_classes <- ifelse(api_options[["census_return_type"]] == 'geographies',
        c('state_fips' = 'character',
          'county_fips' = 'character',
          'census_tract' = 'character',
@@ -84,8 +84,10 @@ batch_census <- function(unique_addresses,
 # Batch geocoding with geocodio
 # ... are arguments passed from the geo() function
 # https://www.geocod.io/docs/#batch-geocoding
-batch_geocodio <- function(unique_addresses, lat = 'lat', long = 'long', timeout = 20, full_results = FALSE, custom_query = list(),
-verbose = FALSE, api_url = NULL, geocodio_v = 1.6, limit = 1, ...) {
+
+batch_geocodio <- function(unique_addresses, lat = 'lat', long = 'long', timeout = 20, 
+          full_results = FALSE, custom_query = list(), verbose = FALSE, api_url = NULL, 
+        api_options = list(), limit = 1, ...) {
   
   # limit the dataframe to legitimate arguments
   address_df <- unique_addresses[names(unique_addresses) %in% get_generic_parameters('geocodio', address_only = TRUE)]
@@ -103,7 +105,7 @@ verbose = FALSE, api_url = NULL, geocodio_v = 1.6, limit = 1, ...) {
     names(address_list) <- 1:nrow(address_df)
   }
   
-  if (is.null(api_url)) api_url <- get_geocodio_url(geocodio_v)
+  if (is.null(api_url)) api_url <- get_geocodio_url(api_options[["geocodio_v"]], reverse = FALSE, geocodio_hipaa = api_options[["geocodio_hipaa"]])
   # Construct query
   query_parameters <- get_api_query('geocodio', list(limit = limit, api_key = get_key('geocodio')),
                                     custom_parameters = custom_query)
@@ -145,8 +147,8 @@ verbose = FALSE, api_url = NULL, geocodio_v = 1.6, limit = 1, ...) {
 # ... are arguments passed from the geo() function
 # https://developer.here.com/documentation/batch-geocoder/dev_guide/topics/introduction.html
 batch_here <- function(unique_addresses, lat = 'lat', long = 'long', timeout = 20, full_results = FALSE, custom_query = list(),
-                       verbose = FALSE, api_url = NULL, geocodio_v = 1.6, limit = 1, 
-                       here_request_id = NULL, ...) {
+                       verbose = FALSE, api_url = NULL, limit = 1, 
+                       api_options = list(), ...) {
 
   # https://developer.here.com/documentation/batch-geocoder/dev_guide/topics/quick-start-batch-geocode.html
   # Specific endpoint
@@ -223,7 +225,7 @@ batch_here <- function(unique_addresses, lat = 'lat', long = 'long', timeout = 2
   # Batch timer
   init_process <- Sys.time()
   
-  if (!is.null(here_request_id)){
+  if (!is.null(api_options[["here_request_id"]])){
     if (verbose) message("HERE: Requesting a previous job")
     
   } else {
@@ -246,10 +248,10 @@ batch_here <- function(unique_addresses, lat = 'lat', long = 'long', timeout = 2
     }
     
     # Retrieve here_request_id
-    here_request_id <- job_result$Response$MetaInfo$RequestId
+    api_options[["here_request_id"]] <- job_result$Response$MetaInfo$RequestId
   }
   
-  if (verbose) message('HERE: RequestID -> ', here_request_id)
+  if (verbose) message('HERE: RequestID -> ', api_options[["here_request_id"]])
   
   # Step 2: Check job until is done ----
   # https://developer.here.com/documentation/batch-geocoder/dev_guide/topics/job-status.html
@@ -260,7 +262,7 @@ batch_here <- function(unique_addresses, lat = 'lat', long = 'long', timeout = 2
   # HERE Batching takes a while!
   while (!current_status %in% c('cancelled', 'failed', 'completed')) {
     Sys.sleep(3) # Arbitrary, 3sec
-    status <- httr::GET(url = paste0(api_url, '/', here_request_id),
+    status <- httr::GET(url = paste0(api_url, '/', api_options[["here_request_id"]]),
                         query = list(action = 'status',
                                      apiKey = get_key('here'))
     )
@@ -287,7 +289,7 @@ batch_here <- function(unique_addresses, lat = 'lat', long = 'long', timeout = 2
   
   # Delete non-completed jobs and return empty
   if (current_status != 'completed') {
-    delete <- httr::DELETE(url = paste0(api_url, '/', here_request_id),
+    delete <- httr::DELETE(url = paste0(api_url, '/', api_options[["here_request_id"]]),
                            query = list(apiKey = get_key('here')))
     
     if (verbose) message('\nHERE: Batch job failure\n')
@@ -296,7 +298,7 @@ batch_here <- function(unique_addresses, lat = 'lat', long = 'long', timeout = 2
   
   # Step 3: GET results and parse ----
   batch_results <-
-    httr::GET(url = paste0(api_url, '/', here_request_id, '/result'),
+    httr::GET(url = paste0(api_url, '/', api_options[["here_request_id"]], '/result'),
               query = list(apiKey = get_key('here'),
                            outputcompressed = FALSE)
     )
@@ -465,7 +467,7 @@ batch_tomtom <- function(unique_addresses, lat = 'lat', long = 'long',
 batch_mapquest <-  function(unique_addresses, lat = "lat", long = "long",
                             timeout = 20, full_results = FALSE, custom_query = list(),
                             verbose = FALSE, api_url = NULL, limit = 1, 
-                            mapquest_open = FALSE, ...) {
+                            api_options = list(), ...) {
     # limit the dataframe to legitimate arguments
     address_df <- unique_addresses[names(unique_addresses) %in% get_generic_parameters("mapquest", address_only = TRUE)]
 
@@ -480,7 +482,7 @@ batch_mapquest <-  function(unique_addresses, lat = "lat", long = "long",
                      mode = "single", full_results = full_results,
                      custom_query = custom_query, verbose = verbose,
                      api_url = api_url, limit = limit,
-                     mapquest_open = mapquest_open)
+                     mapquest_open = api_options[["mapquest_open"]])
 
       # rename lat/long columns
       names(results)[names(results) == "lat"] <- lat
@@ -492,13 +494,11 @@ batch_mapquest <-  function(unique_addresses, lat = "lat", long = "long",
     # Multiple via POST ----
     # https://developer.mapquest.com/documentation/geocoding-api/batch/post/
     if (is.null(api_url)) {
-      url_domain <- if (mapquest_open) "http://open" else  "http://www"
-      
+      url_domain <- if (api_options[["mapquest_open"]]) "http://open" else  "http://www"
       api_url <- paste0(url_domain, ".mapquestapi.com/geocoding/v1/batch")
     }
 
     # Construct query - for display only
-
     query_parameters <- get_api_query("mapquest", 
                                       list(limit = limit, api_key = get_key("mapquest")),
                                       custom_parameters = custom_query)
@@ -520,7 +520,6 @@ batch_mapquest <-  function(unique_addresses, lat = "lat", long = "long",
     )
 
     ## Query API ----
-
     query_results <- query_api(api_url, query_parameters, mode = "list",
                                input_list = address_list, timeout = timeout)
 
