@@ -20,4 +20,14 @@ pkg.globals$geographic_limitations <- list(
 )
 
 # these methods do not support reverse geocoding
-pkg.globals$no_reverse_methods <- c('census')
\ No newline at end of file
+pkg.globals$no_reverse_methods <- c('census')
+
+# default settings for the `api_options` argument in geo() and reverse_geo()
+pkg.globals$default_api_options <- list(
+  census_return_type = "locations", 
+  iq_region = "us", 
+  geocodio_v = 1.6, 
+  geocodio_hipaa = FALSE,
+  mapbox_permanent = FALSE, 
+  mapquest_open = FALSE
+  )
